# -*- coding: utf-8 -*-
# @Time    : 26/08/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : IMEX_init.py
# @Software: PyCharm

from .lib_py_base import DSS_xlsx_save
from .IMEX_to_DSS.xlsx_DSS_xlsx.base_xlsx_DSS import class_xlsx_to_DSS
from .logg_print_alert.logg_alert import add_DSS_empty

aux_xlsx = class_xlsx_to_DSS()

class xlsx_DSS_xlsx:

    def create_template_xlsx(
            self, path_save: str = None, prj_name: str = 'default', elem_list: list = [], all_elem: bool = True):
        """
        Generate .xlsx template for OpenDSS xlsx_data entry.

        :param path_save: Path to save the file created. Default is 'None'.
        :param prj_name: Name of the project or case. Default is 'default'.
        :param elem_list: List of OpenDSS elements considered in xlsx template. Default is '[]'.
        :param all_elem: All OpenDSS elements in the xlsx template. Default is 'True'.
        :return: xlsx file
        """
        aux_xlsx._template_xlsx(path_save, prj_name, elem_list, all_elem)

    def xlsx_to_OpenDSS(
            self, xlsx_path: str = None, path_save: str = None, prj_name: str = 'default', add_path: bool = False,
            coords: str = 'LatLong'):
        """
        Generate OpenDSS files, according to the information found in the xlsx template.

        :param xlsx_path: xlsx file path. Default is 'None'.
        :param path_save: Path to save the file created. Default is 'None'
        :param prj_name: Name of the project or case. Default is 'default'
        :param add_path: Add in the OpenDSS Master the path of the file. Default is 'False'
        :param coords: Type of circuit coordinates, can be 'XY' or 'LatLong'. Default is 'XY'
        :return: DSS files
        """
        aux_xlsx._create_DSS_from_xlsx(xlsx_path, path_save, prj_name, add_path, coords)

    def OpenDSS_to_xlsx(
            self, DSS_path: str = None, path_save: str = None, prj_name: str = 'default',
            add_empty: bool = add_DSS_empty, coords: str = 'LatLong'):
        """
        Generates .xlsx template with xlsx_data from OpenDSS scripts

        :param DSS_path: OpenDSS file path. Default is 'None'
        :param path_save: Path to save the file created. Default is 'None'
        :param prj_name: Name of the project or case. Default is 'default'
        :param add_empty: All OpenDSS elements in the xlsx template. Default is 'True'
        :return:
        """
        aux_xlsx._create_from_DSS_scripts_to_xlsx(DSS_path, path_save, prj_name, add_empty, coords)



# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : PD_elements_MTY.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_PD_elem
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_ORD, _COL_MTY

list_PD_elements_DSS = ['Transformer', 'Line', 'Capacitor', 'AutoTrans', 'GICTransformer', 'Reactor']

def PD_elem_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:

    if name_class == 'Transformer':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():

                if DF_elem_DSS['windings'][index] == 2:
                    if DF_elem_DSS['XHL'][index] != '':
                        row_aux = ['XHT', 'XLT', 'X12', 'X13', 'X23']
                        for x in row_aux:
                            DF_elem_DSS[x][index] = ''

                if DF_elem_DSS['windings'][index] == 3:
                    if DF_elem_DSS['XHL'][index] != '':
                        row_aux = ['X12', 'X13', 'X23']
                        for x in row_aux:
                            DF_elem_DSS[x][index] = ''

                row_aux = ['wdg', 'bus', 'conn', 'kV', 'kVA', 'tap', '%R']
                for x in row_aux:
                    DF_elem_DSS[x][index] = ''



                row_aux = [
                    'Rneut', 'flrise', 'hsrise', '%noloadloss', 'normhkVA', 'emerghkVA',
                    'sub', 'MaxTap', 'MinTap', 'NumTaps', 'subname', '%imag', 'ppm_antifloat', 'XRConst', 'LeadLag',
                    'RdcOhms', 'faultrate', 'basefreq', 'enabled', 'Seasons', 'Ratings', 'n', 'm', 'thermal']

                value_aux = [
                    -1, 65, 15, 0, '110% of kVA rating of Winding 1', '150% of kVA rating of Winding 1', 'No',
                    1.1, 0.9, 32, 'null', 0, 1, 'No', 'Lag', '85% of %R', 0.007, 'basefreq', 'Yes', 1, '[1100.0]',
                    0.8, 0.8, 2]

                for x in zip(row_aux, value_aux):

                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''

        return DF_elem_DSS
    elif name_class == 'Line':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                if DF_elem_DSS['linecode'][index] != '':
                    list_Line = ['rmatrix', 'xmatrix', 'cmatrix']
                    for m in list_Line:
                        DF_elem_DSS[m][index] = ''

                if DF_elem_DSS['Switch'][index] == 'Yes':

                    list_Line = [
                        'r1', 'x1', 'r0', 'x0', 'C1', 'C0', 'rmatrix', 'xmatrix', 'cmatrix', 'Rg', 'Xg', 'rho',
                        'geometry', 'units', 'spacing', 'wires', 'EarthModel', 'cncables', 'tscables', 'B1', 'B0',
                        'Seasons', 'Ratings', 'LineType', 'normamps', 'emergamps', 'faultrate', 'pctperm', 'repair',
                        'basefreq', 'enabled', 'like']
                    for m in list_Line:
                        DF_elem_DSS[m][index] = ''

                    colum_aux = ['r1', 'r0', 'x1', 'x0', 'C1', 'C0']
                    Value = [0.0001, 0.0001, 0, 0, 0, 0]
                    for x in zip(colum_aux, Value):
                        DF_elem_DSS[x[0]][index] = x[1]

                row_aux = [
                    'linecode', 'length', 'Switch', 'Rg', 'Xg', 'rho', 'units', 'EarthModel', 'LineType', 'faultrate',
                    'pctperm', 'repair', 'enabled', 'basefreq']

                value_aux = [
                    '336 MCM ACSR on 4 ft spacing', 1, 'no', 0.01805, 0.155081, 100, 'None', 'Deri', 'OH', 0.1, 20, 3,
                    'Yes', 'base frequency']

                for x in zip(row_aux, value_aux):

                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''

        return DF_elem_DSS

    elif name_class == 'Capacitor':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                row_aux = [
                    'bus2', 'R', 'Harm', 'Numsteps', 'states', 'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled']

                value_aux = [
                    'bus1.0', 0, 0, 1, '[1.0]', 0.0005, 100, 3, 'BaseFreq', 'Yes']

                for x in zip(row_aux, value_aux):

                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''


        return DF_elem_DSS

    elif name_class == 'AutoTrans':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'GICTransformer':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    elif name_class == 'Reactor':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS
    else:
        return DF_elem_DSS

def PD_elements_ORD(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:

    if name_class == 'Transformer':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    elif name_class == 'Line':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    elif name_class == 'Capacitor':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    elif name_class == 'AutoTrans':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    elif name_class == 'GICTransformer':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    elif name_class == 'Reactor':
        return _COL_ORD(dict_PD_elem, DF_elem_DSS, name_class)
    else:
        return DF_elem_DSS

def PD_elements_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):

    #list_property = dss.dsselement_all_property_names()
    list_property = dict_PD_elem[name_class]

    if name_class == 'Transformer':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Line':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Capacitor':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'AutoTrans':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'GICTransformer':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Reactor':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return BBDD_elem_DSS, DSS_elem_list


# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : Other_elem_DSS.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_Other
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_MTY, _COL_ORD

list_Other_DSS = ['Vsource', 'Fault', 'GICsource', 'Isource']

def Other_ORD(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:
    if name_class == 'Vsource':
        return _COL_ORD(dict_Other, DF_elem_DSS, name_class)

    elif name_class == 'Fault':
        return _COL_ORD(dict_Other, DF_elem_DSS, name_class)

    elif name_class == 'GICsource':
        return _COL_ORD(dict_Other, DF_elem_DSS, name_class)

    elif name_class == 'Isource':
        return _COL_ORD(dict_Other, DF_elem_DSS, name_class)

    else:
        return DF_elem_DSS

def Other_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:
    if name_class == 'Vsource':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                if DF_elem_DSS['MVAsc3'][index] != '' and DF_elem_DSS['MVAsc1'][index] != '':
                    list_Vs = ['x1r1', 'x0r0',
                               'Isc3', 'Isc1',
                               'R1', 'X1', 'R0', 'X0', 'Z1', 'Z0', 'Z2',
                               'puZ1', 'puZ0', 'puZ2']
                    for m in list_Vs:
                        DF_elem_DSS[m][index] = ''

                if DF_elem_DSS['Isc3'][index] != '' and DF_elem_DSS['Isc1'][index] != '':
                    list_Vs = ['x1r1', 'x0r0',
                               'MVAsc3', 'MVAsc1',
                               'R1', 'X1', 'R0', 'X0', 'Z1', 'Z0', 'Z2',
                               'puZ1', 'puZ0', 'puZ2']
                    for m in list_Vs:
                        DF_elem_DSS[m][index] = ''

                if DF_elem_DSS['x1r1'][index] != '' and DF_elem_DSS['x0r0'][index] != '':
                    list_Vs = ['Isc3', 'Isc1',
                               'MVAsc3', 'MVAsc1',
                               'R1', 'X1', 'R0', 'X0', 'Z1', 'Z0', 'Z2',
                               'puZ1', 'puZ0', 'puZ2']
                    for m in list_Vs:
                        DF_elem_DSS[m][index] = ''


                row_aux = [
                    'frequency', 'MVAsc3', 'MVAsc1', 'x1r1', 'x0r0', 'Isc3', 'Isc1', 'R1', 'X1', 'R0', 'X0',
                    'ScanType', 'Sequence', 'bus2', 'baseMVA', 'Model', 'puZideal', 'spectrum', 'enabled', 'basefreq']
                
                value_aux = [
                    'base frequency', 2000, 2100, 4, 3, 1000, 10500, 1.65, 6.6, 1.9, 5.7, 'Positive', 'Positive',
                    'Bus1.0.0.0', 100, 'Thevenin', '[1e-06, 0.001]', 'defaultvsource', 'Yes', 'base frequency']

                for x in zip(row_aux, value_aux):
                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == f'{DF_elem_DSS["bus1"][index]}.0.0.0':
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == f'{DF_elem_DSS["bus1"][index]}.0.0':
                        DF_elem_DSS[x[0]][index] = ''


        return DF_elem_DSS

    elif name_class == 'Fault':
        if not DF_elem_DSS.empty:
            pass

        return DF_elem_DSS

    elif name_class == 'GICsource':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'Isource':
        if not DF_elem_DSS.empty:
            pass

        return DF_elem_DSS

    else:
        return DF_elem_DSS

def Other_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):
    #list_property = dss.dsselement_all_property_names()
    list_property = dict_Other[name_class]

    if name_class == 'Vsource':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Fault':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'GICsource':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Isource':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)


# -*- coding: utf-8 -*-
# @Time    : 8/23/2022 3:41 PM
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : lib_py_base.py
# @Software: PyCharm

class DSS_xlsx_save:

    def __init__(self, DSS_path: str = None, xlsx_path: str = None, path_save: str = None):
        self.DSS_path = DSS_path
        self.xlsx_path = xlsx_path
        self.path_save = path_save





# -*- coding: utf-8 -*-
# @Time    : 11/11/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : NameClass_columns.py
# @Software: PyCharm


list_General_DSS = ['WireData', 'LineSpacing', 'LineGeometry', 'LineCode', 'XfmrCode', 'CNData', 'GrowthShape',
                    'LoadShape', 'PriceShape', 'Spectrum', 'TCC_Curve', 'TSData', 'TShape', 'XYcurve']

dict_General = dict()
dict_General['LineCode'] = [
    'nphases', 'r1', 'x1', 'r0', 'x0', 'C1', 'C0', 'units', 'rmatrix', 'xmatrix', 'cmatrix', 'baseFreq', 'normamps',
    'emergamps', 'faultrate', 'pctperm', 'repair', 'Kron', 'Rg', 'Xg', 'rho', 'neutral', 'B1', 'B0', 'Seasons',
    'Ratings', 'LineType', 'like']
dict_General['LoadShape'] = [
    'npts', 'interval', 'mult', 'hour', 'mean', 'stddev', 'csvfile', 'sngfile', 'dblfile', 'action', 'qmult',
    'UseActual', 'Pmax', 'Qmax', 'sinterval', 'minterval', 'Pbase', 'Qbase', 'Pmult', 'PQCSVFile', 'MemoryMapping',
    'like']
dict_General['TShape'] = [
    'npts', 'interval', 'temp', 'hour', 'mean', 'stddev', 'csvfile', 'sngfile', 'dblfile', 'sinterval', 'minterval',
    'action', 'like']
dict_General['PriceShape'] = [
    'npts', 'interval', 'price', 'hour', 'mean', 'stddev', 'csvfile', 'sngfile', 'dblfile', 'sinterval', 'minterval',
    'action', 'like']
dict_General['XYcurve'] = [
    'npts', 'Points', 'Yarray', 'Xarray', 'csvfile', 'sngfile', 'dblfile', 'x', 'y', 'Xshift', 'Yshift', 'Xscale',
    'Yscale', 'like']
dict_General['GrowthShape'] = ['npts', 'year', 'mult', 'csvfile', 'sngfile', 'dblfile', 'like']
dict_General['TCC_Curve'] = ['npts', 'C_array', 'T_array', 'like']
dict_General['Spectrum'] = ['NumHarm', 'harmonic', '%mag', 'angle', 'CSVFile', 'like']
dict_General['WireData'] = ['Rdc', 'Rac', 'Runits', 'GMRac', 'GMRunits', 'radius', 'radunits', 'normamps', 'emergamps',
                            'diam', 'Seasons', 'Ratings', 'Capradius', 'like']
dict_General['CNData'] = [
    'k', 'DiaStrand', 'GmrStrand', 'Rstrand', 'EpsR', 'InsLayer', 'DiaIns', 'DiaCable', 'Rdc', 'Rac', 'Runits', 'GMRac',
    'GMRunits', 'radius', 'radunits', 'normamps', 'emergamps', 'diam', 'Seasons', 'Ratings', 'Capradius', 'like']
dict_General['TSData'] = [
    'DiaShield', 'TapeLayer', 'TapeLap', 'EpsR', 'InsLayer', 'DiaIns', 'DiaCable', 'Rdc', 'Rac', 'Runits', 'GMRac',
    'GMRunits', 'radius', 'radunits', 'normamps', 'emergamps', 'diam', 'Seasons', 'Ratings', 'Capradius', 'like']
dict_General['LineGeometry'] = [
    'nconds', 'nphases', 'cond', 'wire', 'x', 'h', 'units', 'normamps', 'emergamps', 'reduce', 'spacing', 'wires',
    'cncable', 'tscable', 'cncables', 'tscables', 'Seasons', 'Ratings', 'LineType', 'like']
dict_General['LineSpacing'] = ['nconds', 'nphases', 'x', 'h', 'units', 'like']
dict_General['XfmrCode'] = [
    'phases', 'windings', 'wdg', 'conn', 'kV', 'kVA', 'tap', '%R', 'Rneut', 'Xneut', 'conns', 'kVs', 'kVAs', 'taps',
    'Xhl', 'Xht', 'Xlt', 'Xscarray', 'thermal', 'n', 'm', 'flrise', 'hsrise', '%loadloss', '%noloadloss', 'normhkVA',
    'emerghkVA', 'MaxTap', 'MinTap', 'NumTaps', '%imag', 'ppm_antifloat', '%Rs', 'X12', 'X13', 'X23', 'RdcOhms',
    'Seasons', 'Ratings', 'like']

list_Other_DSS = ['Vsource', 'Fault', 'GICsource', 'Isource']
dict_Other = dict()

dict_Other['Vsource'] = [
    'bus1', 'basekv', 'pu', 'angle', 'frequency', 'phases', 'MVAsc3', 'MVAsc1', 'x1r1', 'x0r0', 'Isc3', 'Isc1', 'R1',
    'X1', 'R0', 'X0', 'ScanType', 'Sequence', 'bus2', 'Z1', 'Z0', 'Z2', 'puZ1', 'puZ0', 'puZ2', 'baseMVA', 'Yearly',
    'Daily', 'Duty', 'Model', 'puZideal', 'spectrum', 'basefreq', 'enabled', 'like']
dict_Other['Fault'] = [
    'bus1', 'bus2', 'phases', 'r', '%stddev', 'Gmatrix', 'ONtime', 'temporary', 'MinAmps', 'normamps', 'emergamps',
    'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled', 'like']
dict_Other['GICsource'] = [
    'Description', 'Volts', 'angle', 'frequency', 'phases', 'EN', 'EE', 'Lat1', 'Lon1', 'Lat2', 'Lon2', 'spectrum',
    'basefreq', 'enabled', 'like']
dict_Other['Isource'] = [
    'bus1', 'amps', 'angle', 'frequency', 'phases', 'scantype', 'sequence', 'Yearly', 'Daily', 'Duty', 'Bus2',
    'spectrum', 'basefreq', 'enabled', 'like']

list_PD_elements_DSS = ['Transformer', 'Line', 'Capacitor', 'AutoTrans', 'GICTransformer', 'Reactor']
dict_PD_elem = dict()

dict_PD_elem['Transformer'] = [
    'phases', 'windings', 'XfmrCode', 'buses', 'conns', 'kVs', 'kVAs', 'taps', '%Rs', 'MaxTap', 'MinTap',
    'NumTaps',  'normamps', 'emergamps', 'normhkVA', 'emerghkVA', 'wdg', 'bus', 'conn', 'kV', 'kVA', 'tap', '%R',
    'Rneut', 'Xneut', 'RdcOhms', 'XHL', 'XHT', 'XLT', 'X12', 'X13', 'X23', '%loadloss', '%noloadloss', 'Xscarray',
    'thermal', 'n', 'm', 'flrise', 'hsrise', 'sub', 'subname', '%imag', 'ppm_antifloat', 'bank', 'XRConst', 'LeadLag',
    'WdgCurrents', 'Core', 'Seasons', 'Ratings', 'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled', 'like']
dict_PD_elem['Line'] = [
    'bus1', 'bus2', 'linecode', 'length', 'phases', 'r1', 'x1', 'r0', 'x0', 'C1', 'C0', 'rmatrix', 'xmatrix', 'cmatrix',
    'Switch', 'Rg', 'Xg', 'rho', 'geometry', 'units', 'spacing', 'wires', 'EarthModel', 'cncables', 'tscables', 'B1',
    'B0', 'Seasons', 'Ratings', 'LineType', 'normamps', 'emergamps', 'faultrate', 'pctperm', 'repair', 'basefreq',
    'enabled', 'like']
dict_PD_elem['Capacitor'] = [
    'bus1', 'bus2', 'phases', 'kvar', 'kv', 'conn', 'cmatrix', 'cuf', 'R', 'XL', 'Harm', 'Numsteps', 'states',
    'normamps', 'emergamps', 'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled', 'like']
dict_PD_elem['AutoTrans'] = [
    'phases', 'windings', 'wdg', 'bus', 'conn', 'kV', 'kVA', 'tap', '%R', 'Rdcohms', 'Core', 'buses', 'conns', 'kVs',
    'kVAs', 'taps', 'XHX', 'XHT', 'XXT', 'XSCarray', 'thermal', 'n', 'm', 'flrise', 'hsrise', '%loadloss',
    '%noloadloss', 'normhkVA', 'emerghkVA', 'sub', 'MaxTap', 'MinTap', 'NumTaps', 'subname', '%imag', 'ppm_antifloat',
    '%Rs', 'bank', 'XfmrCode', 'XRConst', 'LeadLag', 'WdgCurrents', 'normamps', 'emergamps', 'faultrate', 'pctperm',
    'repair', 'basefreq', 'enabled', 'like']
dict_PD_elem['GICTransformer'] = [
    'BusH', 'BusNH', 'BusX', 'BusNX', 'phases', 'Type', 'R1', 'R2', 'KVLL1', 'KVLL2', 'MVA', 'VarCurve', '%R1', '%R2',
    'K', 'normamps', 'emergamps', 'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled', 'like']
dict_PD_elem['Reactor'] = [
    'bus1', 'bus2', 'phases', 'kvar', 'kv', 'conn', 'Rmatrix', 'Xmatrix', 'Parallel', 'R', 'X', 'Rp', 'Z1', 'Z2', 'Z0',
    'Z', 'RCurve', 'LCurve', 'LmH', 'normamps', 'emergamps', 'faultrate', 'pctperm', 'repair', 'basefreq', 'enabled',
    'like']

list_PC_elements_DSS = [
    'Generator', 'Generic5', 'GICLine', 'IndMach012', 'Load', 'PVSystem', 'Storage', 'UPFC', 'VCCS', 'VSConverter',
    'WindGen']
dict_PC_elem = dict()
dict_PC_elem['Generator'] = [
    'phases', 'bus1', 'kv', 'kW', 'pf', 'kvar', 'model', 'Vminpu', 'Vmaxpu', 'yearly', 'daily', 'duty', 'dispmode',
    'dispvalue', 'conn', 'Rneut', 'Xneut', 'status', 'class', 'Vpu', 'maxkvar', 'minkvar', 'pvfactor', 'forceon', 'kVA',
    'MVA', 'Xd', 'Xdp', 'Xdpp', 'H', 'D', 'UserModel', 'UserData', 'ShaftModel', 'ShaftData', 'DutyStart', 'debugtrace',
    'Balanced', 'XRdp', 'UseFuel', 'FuelkWh', '%Fuel', '%Reserve', 'Refuel', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['Generic5'] = [
    'phases', 'bus1', 'kv', 'kW', 'pf', 'conn', 'kVA', 'H', 'D', 'P_ref1kW', 'P_ref2kW', 'P_ref3kW', 'V_ref1kVLN',
    'V_ref2kVLN', 'V_ref3kVLN', 'MaxSlip', 'SlipOption', 'Yearly', 'Daily', 'Duty', 'Debugtrace', 'P_refkW',
    'Q_refkVAr', 'Cluster_num', 'V_refkVLN', 'ctrl_mode', 'QV_flag', 'kcd', 'kcq', 'kqi', 'Q_ref1kVAr', 'Q_ref2kVAr',
    'Q_ref3kVAr', 'PmaxkW', 'PminkW', 'PQpriority', 'PmppkW', 'Pfctr1', 'Pfctr2', 'Pfctr3', 'Pfctr4', 'Pfctr5',
    'Pfctr6', 'PbiaskW', 'CC_Switch', 'kcq_drp2', 'Volt_Trhd', 'droop', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['GICLine'] = [
    'bus1', 'bus2', 'Volts', 'Angle', 'frequency', 'phases', 'R', 'X', 'C', 'EN', 'EE', 'Lat1', 'Lon1', 'Lat2', 'Lon2',
    'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['IndMach012'] = [
    'phases', 'bus1', 'kv', 'kW', 'pf', 'conn', 'kVA', 'H', 'D', 'puRs', 'puXs', 'puRr', 'puXr', 'puXm', 'Slip',
    'MaxSlip', 'SlipOption', 'Yearly', 'Daily', 'Duty', 'Debugtrace', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['Load'] = [
    'phases', 'bus1', 'kV', 'kW', 'kvar', 'kVA', 'pf', 'model', 'yearly', 'daily', 'duty', 'growth', 'conn',
    'Rneut', 'Xneut', 'status', 'class', 'Vminpu', 'Vmaxpu', 'Vminnorm', 'Vminemerg', 'xfkVA', 'allocationfactor',
    '%mean', '%stddev', 'CVRwatts', 'CVRvars', 'kwh', 'kwhdays', 'Cfactor', 'CVRcurve', 'NumCust', 'ZIPV', '%SeriesRL',
    'RelWeight', 'Vlowpu', 'puXharm', 'XRharm', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['PVSystem'] = [
    'phases', 'bus1', 'kv', 'irradiance', 'Pmpp', '%Pmpp', 'Temperature', 'pf', 'conn', 'kvar', 'kVA', '%Cutin',
    '%Cutout', 'EffCurve', 'P-TCurve', '%R', '%X', 'model', 'Vminpu', 'Vmaxpu', 'Balanced', 'LimitCurrent', 'yearly',
    'daily', 'duty', 'Tyearly', 'Tdaily', 'Tduty', 'class', 'UserModel', 'UserData', 'debugtrace', 'VarFollowInverter',
    'DutyStart', 'WattPriority', 'PFPriority', '%PminNoVars', '%PminkvarMax', 'kvarMax', 'kvarMaxAbs', 'spectrum',
    'basefreq', 'enabled', 'like']
dict_PC_elem['Storage'] = [
    'phases', 'bus1', 'kv', 'conn', 'kW', 'kvar', 'pf', 'kVA', '%Cutin', '%Cutout', 'EffCurve', 'VarFollowInverter',
    'kvarMax', 'kvarMaxAbs', 'WattPriority', 'PFPriority', '%PminNoVars', '%PminkvarMax', 'kWrated', '%kWrated',
    'kWhrated', 'kWhstored', '%stored', '%reserve', 'State', '%Discharge', '%Charge', '%EffCharge', '%EffDischarge',
    '%IdlingkW', '%Idlingkvar', '%R', '%X', 'model', 'Vminpu', 'Vmaxpu', 'Balanced', 'LimitCurrent', 'yearly', 'daily',
    'duty', 'DispMode', 'DischargeTrigger', 'ChargeTrigger', 'TimeChargeTrig', 'class', 'DynaDLL', 'DynaData',
    'UserModel', 'UserData', 'debugtrace', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['UPFC'] = [
    'bus1', 'bus2', 'refkv', 'pf', 'frequency', 'phases', 'Xs', 'Tol1', 'Mode', 'VpqMax', 'LossCurve', 'VHLimit',
    'VLLimit', 'CLimit', 'refkv2', 'kvarLimit', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['VCCS'] = [
    'bus1', 'phases', 'prated', 'vrated', 'ppct', 'bp1', 'bp2', 'filter', 'fsample', 'rmsmode', 'imaxpu', 'vrmstau',
    'irmstau', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['VSConverter'] = [
    'phases', 'Bus1', 'kVac', 'kVdc', 'kW', 'Ndc', 'Rac', 'Xac', 'm0', 'd0', 'Mmin', 'Mmax', 'Iacmax', 'Idcmax',
    'Vacref', 'Pacref', 'Qacref', 'Vdcref', 'VscMode', 'spectrum', 'basefreq', 'enabled', 'like']
dict_PC_elem['WindGen'] = [
    'phases', 'bus1', 'kv', 'kW', 'PF', 'model', 'yearly', 'daily', 'duty', 'conn', 'kvar', 'status', 'class', 'Vpu',
    'maxkvar', 'minkvar', 'pvfactor', 'debugtrace', 'Vminpu', 'Vmaxpu', 'forceon', 'kVA', 'MVA', 'Xd', 'Xdp', 'Xdpp',
    'H', 'D', 'UserModel', 'UserData', 'ShaftModel', 'ShaftData', 'DutyStart', 'Balanced', 'XRdp', 'spectrum',
    'basefreq', 'enabled', 'like']

list_Controls_DSS = ['CapControl', 'ESPVLControl', 'ExpControl', 'Fuse', 'GenDispatcher', 'InvControl', 'Recloser',
                     'RegControl', 'Relay', 'StorageController', 'SwtControl', 'UPFCControl']
dict_Control = dict()
dict_Control['CapControl'] = [
    'element', 'terminal', 'capacitor', 'type', 'PTratio', 'CTratio', 'ONsetting', 'OFFsetting', 'Delay',
    'VoltOverride', 'Vmax', 'Vmin', 'DelayOFF', 'DeadTime', 'CTPhase', 'PTPhase', 'VBus', 'EventLog', 'UserModel',
    'UserData', 'pctMinkvar', 'Reset', 'basefreq', 'enabled', 'like']
dict_Control['ESPVLControl'] = [
    'Element', 'Terminal', 'Type', 'kWBand', 'kvarlimit', 'LocalControlList', 'LocalControlWeights', 'PVSystemList',
    'PVSystemWeights', 'StorageList', 'StorageWeights', 'Forecast', 'basefreq', 'enabled', 'like']
dict_Control['ExpControl'] = [
    'PVSystemList', 'Vreg', 'Slope', 'VregTau', 'Qbias', 'VregMin', 'VregMax', 'QmaxLead', 'QmaxLag', 'EventLog',
    'DeltaQ_factor', 'PreferQ', 'Tresponse', 'basefreq', 'enabled', 'like']
dict_Control['Fuse'] = [
    'MonitoredObj', 'MonitoredTerm', 'SwitchedObj', 'SwitchedTerm', 'FuseCurve', 'RatedCurrent', 'Delay', 'Action',
    'Normal', 'State', 'basefreq', 'enabled', 'like']
dict_Control['GenDispatcher'] = [
    'Element', 'Terminal', 'kWLimit', 'kWBand', 'kvarlimit', 'GenList', 'basefreq', 'enabled', 'like']
dict_Control['InvControl'] = [
    'DERList', 'Mode', 'CombiMode', 'vvc_curve1', 'hysteresis_offset', 'voltage_curvex_ref', 'avgwindowlen',
    'voltwatt_curve', 'DbVMin', 'DbVMax', 'ArGraLowV', 'ArGraHiV', 'DynReacavgwindowlen', 'deltaQ_Factor',
    'VoltageChangeTolerance', 'VarChangeTolerance', 'VoltwattYAxis', 'RateofChangeMode', 'LPFTau', 'RiseFallLimit',
    'deltaP_Factor', 'EventLog', 'RefReactivePower', 'ActivePChangeTolerance', 'monVoltageCalc', 'monBus',
    'MonBusesVbase', 'voltwattCH_curve', 'wattpf_curve', 'wattvar_curve', 'VV_RefReactivePower', 'PVSystemList',
    'Vsetpoint', 'basefreq', 'enabled', 'like']
dict_Control['Recloser'] = [
    'MonitoredObj', 'MonitoredTerm', 'SwitchedObj', 'SwitchedTerm', 'NumFast', 'PhaseFast', 'PhaseDelayed',
    'GroundFast', 'GroundDelayed', 'PhaseTrip', 'GroundTrip', 'PhaseInst', 'GroundInst', 'Reset', 'Shots',
    'RecloseIntervals', 'Delay', 'Action', 'TDPhFast', 'TDGrFast', 'TDPhDelayed', 'TDGrDelayed', 'Normal', 'State',
    'basefreq', 'enabled', 'like']
dict_Control['RegControl'] = [
    'transformer', 'winding', 'vreg', 'band', 'ptratio', 'CTprim', 'R', 'X', 'bus', 'delay', 'reversible', 'revvreg',
    'revband', 'revR', 'revX', 'tapdelay', 'debugtrace', 'maxtapchange', 'inversetime', 'tapwinding', 'vlimit',
    'PTphase', 'revThreshold', 'revDelay', 'revNeutral', 'EventLog', 'RemotePTRatio', 'TapNum', 'Reset', 'LDC_Z',
    'rev_Z', 'Cogen', 'basefreq', 'enabled', 'like']
dict_Control['Relay'] = [
    'MonitoredObj', 'MonitoredTerm', 'SwitchedObj', 'SwitchedTerm', 'type', 'Phasecurve', 'Groundcurve', 'PhaseTrip',
    'GroundTrip', 'TDPhase', 'TDGround', 'PhaseInst', 'GroundInst', 'Reset', 'Shots', 'RecloseIntervals', 'Delay',
    'Overvoltcurve', 'Undervoltcurve', 'kvbase', '47%Pickup', '46BaseAmps', '46%Pickup', '46isqt', 'Variable',
    'overtrip', 'undertrip', 'Breakertime', 'action', 'Z1mag', 'Z1ang', 'Z0mag', 'Z0ang', 'Mphase', 'Mground',
    'EventLog', 'DebugTrace', 'DistReverse', 'Normal', 'State', 'basefreq', 'enabled', 'like']
dict_Control['StorageController'] = [
    'Element', 'Terminal', 'MonPhase', 'kWTarget', 'kWTargetLow', '%kWBand', 'kWBand', '%kWBandLow', 'kWBandLow',
    'ElementList', 'Weights', 'ModeDischarge', 'ModeCharge', 'TimeDischargeTrigger', 'TimeChargeTrigger', '%RatekW',
    '%RateCharge', '%Reserve', 'kWhTotal', 'kWTotal', 'kWhActual', 'kWActual', 'kWneed', 'Yearly', 'Daily', 'Duty',
    'EventLog', 'InhibitTime', 'Tup', 'TFlat', 'Tdn', 'kWThreshold', 'DispFactor', 'ResetLevel', 'Seasons',
    'SeasonTargets', 'SeasonTargetsLow', 'basefreq', 'enabled', 'like']
dict_Control['SwtControl'] = [
    'SwitchedObj', 'SwitchedTerm', 'Action', 'Lock', 'Delay', 'Normal', 'State', 'Reset', 'basefreq', 'enabled', 'like']
dict_Control['UPFCControl'] = ['UPFCList', 'basefreq', 'enabled', 'like']

list_Meters_DSS = ['EnergyMeter', 'FMonitor', 'Monitor', 'Sensor']
dict_Meters = dict()

dict_Meters['EnergyMeter'] = [
    'element', 'terminal', 'action', 'option', 'kVAnormal', 'kVAemerg', 'peakcurrent', 'Zonelist', 'LocalOnly', 'Mask',
    'Losses', 'LineLosses', 'XfmrLosses', 'SeqLosses', '3phaseLosses', 'VbaseLosses', 'PhaseVoltageReport', 'Int_Rate',
    'Int_Duration', 'SAIFI', 'SAIFIkW', 'SAIDI', 'CAIDI', 'CustInterrupts', 'basefreq', 'enabled', 'like']
dict_Meters['FMonitor'] = [
    'element', 'terminal', 'mode', 'action', 'residual', 'VIPolar', 'PPolar', 'P_trans_ref', 'V_Sensor', 'P_Sensor',
    'Node_num', 'Cluster_num', 'Total_Clusters', 'Nodes', 'CommVector', 'ElemTableLine', 'P_Mode', 'CommDelayVector',
    'T_intvl_smpl', 'MaxLocalMem', 'Volt_limits_pu', 'b_Curt_Ctrl', 'up_dly', 'virtual_ld_node:', 'EGen',
    'attack_defense', 'Comm_hide', 'Comm_node_hide', 'basefreq', 'enabled', 'like']
dict_Meters['Monitor'] = [
    'element', 'terminal', 'mode', 'action', 'residual', 'VIPolar', 'PPolar', 'basefreq', 'enabled', 'like']
dict_Meters['Sensor'] = [
    'element', 'terminal', 'kvbase', 'clear', 'kVs', 'currents', 'kWs', 'kvars', 'conn', 'Deltadirection', '%Error',
    'Weight', 'action', 'basefreq', 'enabled', 'like']




# -*- coding: utf-8 -*-
# @Time    : 26/10/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : DSS_xlsx_save.py
# @Software: PyCharm

import logging
import os
import pathlib
from ...lib_py_base import DSS_xlsx_save
from .DSS_script import create_scrips_base_dss
from openpy_fx_tools_dss.logg_print_alert import logg_alert
from .create_file_xlsx import _Create_DSS_to_xlsx_files, _Create_template_xlsx

log_py = logging.getLogger(__name__)

class class_xlsx_to_DSS:

    def _template_xlsx(self, path_save: str, prj_name: str, elem: list, all_elem: bool):
        """
        creates xlsx template in the path_save entered

        :return: xlsx file
        """
        if path_save is None:
            logg_alert.update_logg_file('Enter the path to save the .xlsx file', 4, log_py)
        else:
            if prj_name == 'default':
                logg_alert.update_logg_file(
                    'The file has a "default" name. You can customize the file name, see documentation.', 3, log_py)
            _Create_template_xlsx(path_save, prj_name, elem, all_elem)
            print('-' * 64)
            logg_alert.update_logg_file(
                f'The file BBDD_DSS_{prj_name}.xlsx was successfully created and is located at:', 2, log_py)
            logg_alert.update_logg_file(f'{path_save}', 1, log_py)

    def _create_DSS_from_xlsx(self, xlsx_path: str, path_save: str, prj_name: str, path: bool, coords: str):
        """
        Generate OpenDSS files, according to the information found in the xlsx template.

        :return: DSS files
        """
        if prj_name is None:
            prj_name = 'default'
            logg_alert.alert_messages('OpenDSS scripts are called ClassName_default.DSS', 3)
        create_scrips_base_dss(
            name_dss=prj_name,
            xlsx_file=xlsx_path,
            path_save=path_save,
            add_path=path,
            coords=coords)
        logg_alert.update_logg_file(f'The .DSS files are saved in:\n {path_save}')


    def _create_from_DSS_scripts_to_xlsx(
            self, DSS_path: str, path_save: str, prj_name: str, add_empty: bool, coords: str):

        aux_save = path_save is None
        aux_DSS = DSS_path is None

        if aux_save and aux_DSS:
            logg_alert.update_logg_file(
                'You must indicate the path_save to save the .xlsx file and path_save to OpenDSS files.', 4, log_py)
            exit()
        elif aux_save:
            logg_alert.update_logg_file('You must indicate the path_save to save the .xlsx file.', 4, log_py)
            exit()
        elif aux_DSS:
            logg_alert.update_logg_file('You must indicate the path_save to OpenDSS files', 4, log_py)
            exit()
        else:
            _Create_DSS_to_xlsx_files(DSS_path, path_save, prj_name, add_empty, coords)





# -*- coding: utf-8 -*-
# @Time    : 15/11/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : General_elem_DSS.py
# @Software: PyCharm

import pandas as pd

def _COL_ORD(dict_obj: dict, DF_elem_DSS: pd.DataFrame, name_class: str):
    list_aux = [f'Id_{name_class}']
    list_aux = list_aux + dict_obj[name_class]
    DF_elem_DSS = DF_elem_DSS[list_aux]
    return DF_elem_DSS

def _COL_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str, list_property: list):
    list_aux = [f'Id_{name_class}']
    list_aux = list_aux + list_property
    df_elem_DSS = pd.DataFrame(columns=list_aux)
    BBDD_elem_DSS[name_class] = df_elem_DSS
    DSS_elem_list.append(name_class)
    return BBDD_elem_DSS, DSS_elem_list


# -*- coding: utf-8 -*-
# @Time    : 25/09/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : PC_elements_MTY.py
# @Software: PyCharm

import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss
from openpy_fx_tools_dss.NameClass_columns import dict_PC_elem
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.fx_objects import _COL_ORD, _COL_MTY


list_PC_elements_DSS = ['Load', 'Generator', 'Generic5', 'GICLine', 'IndMach012', 'PVSystem', 'UPFC', 'VCCS', 'Storage',
                        'VSConverter', 'WindGen']

def PC_elem_Def_Value(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:
    """

    :param DF_elem_DSS:
    :param name_class:
    :return:
    """
    if name_class == 'Load':
        if not DF_elem_DSS.empty:
            for index, row in DF_elem_DSS.iterrows():
                if DF_elem_DSS['kW'][index] != '' and DF_elem_DSS['kvar'][index] != '':
                        list_Vs = ['pf', 'kVA']

                        for m in list_Vs:
                            DF_elem_DSS[m][index] = ''

                row_aux = ['yearly', 'daily', 'Rneut', 'status', 'class', 'Vminpu', 'Vmaxpu',
                           'Vminnorm', 'Vminemerg', 'xfkVA', 'allocationfactor', '%mean', '%stddev', 'CVRwatts',
                           'CVRvars', 'kwh', 'kwhdays', 'Cfactor', 'CVRcurve', 'NumCust', '%SeriesRL', 'RelWeight',
                           'Vlowpu', 'puXharm', 'XRharm', 'spectrum', 'enabled', 'ZIPV', 'basefreq']

                value_aux = ['no variation', 'no variation', -1, 'Variable', 1, 0.95, 1.05, 0, 0, 0, 0.5, 50,
                             10, 1, 2, 0, 30, 4, 'none', 1, 50, 1, 0.5, 0, 6, 'defaultload', 'Yes', '[0 0 0 0 0 0 0]', 'basefreq']
                for x in zip(row_aux, value_aux):
                    if DF_elem_DSS[x[0]][index] == dss.solution_read_frequency():
                        DF_elem_DSS[x[0]][index] = ''

                    if DF_elem_DSS[x[0]][index] == x[1]:
                        DF_elem_DSS[x[0]][index] = ''

        return DF_elem_DSS

    elif name_class == 'Generator':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'Generic5':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'GICLine':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'IndMach012':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'PVSystem':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'UPFC':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'VCCS':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'Storage':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'VSConverter':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    elif name_class == 'WindGen':
        if not DF_elem_DSS.empty:
            pass
        return DF_elem_DSS

    else:
        return DF_elem_DSS



def PC_elements_ORD(DF_elem_DSS: pd.DataFrame, name_class: str) -> pd.DataFrame:

    if name_class == 'Load':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'Generator':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'Generic5':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'GICLine':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'IndMach012':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'PVSystem':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'UPFC':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'VCCS':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'Storage':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'VSConverter':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)

    elif name_class == 'WindGen':
        return _COL_ORD(dict_PC_elem, DF_elem_DSS, name_class)
    else:
        return DF_elem_DSS

def PC_elements_MTY(BBDD_elem_DSS: dict, DSS_elem_list: list, name_class: str):

    #list_property = dss.dsselement_all_property_names()
    list_property = dict_PC_elem[name_class]

    if name_class == 'Load':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Generator':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Generic5':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'GICLine':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'IndMach012':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'PVSystem':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'UPFC':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'VCCS':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'Storage':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'VSConverter':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    elif name_class == 'WindGen':
        return _COL_MTY(BBDD_elem_DSS, DSS_elem_list, name_class, list_property)
    else:
        return BBDD_elem_DSS, DSS_elem_list



# -*- coding: utf-8 -*-
# @Time    : 22/10/2021
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import pandas as pd
import os.path
import openpyxl
import logging


def characters_delete(DataFrame: pd.DataFrame, column: str) -> object:
    """
    Function that eliminates empty spaces and special characters that can cause convergence problems in OpenDSS

    :param DataFrame: Name of the selected dataFrame
    :param column: Column name of the selected dataFrame
    :return:
    """
    characters_delete_space = [' ', '  ', '   ']
    for i in characters_delete_space:
        DataFrame[column] = DataFrame[column].apply(str).str.replace(i, '', regex=True)

    characters_delete = ['-', '/', '*', '.', '(', ')', 'á', 'é', 'í', 'ó', 'ú']
    for k in characters_delete:
        DataFrame[column] = DataFrame[column].apply(str).str.replace(k, '_', regex=True)
        # DataFrame[column] = DataFrame[column].str.replace(k, '', regex=True)


def characters_delete_phase(DataFrame: pd.DataFrame, column: str):
    DataFrame[column] = DataFrame[column].astype(str)
    DataFrame[column] = DataFrame[column].str.replace('.1', '')
    DataFrame[column] = DataFrame[column].str.replace('.2', '')
    DataFrame[column] = DataFrame[column].str.replace('.3', '')
    DataFrame[column] = DataFrame[column].str.replace('.4', '')


def characters_delete_phase_2(DataFrame: pd.DataFrame, column: str):
    characters_delete = ['.1', '.2', '.3', '.4']
    # characters_delete = ['.1.2.3.4', '.1.2.3', '.1', '.2', '.3', '.4']
    for k in characters_delete:
        DataFrame[column] = DataFrame[column].str.replace(k, '')
        # DataFrame[column] = DataFrame[column].str.replace(k, '', regex=True)
        # DataFrame[column] = DataFrame[column].apply(str).str.replace(k, '', regex=True)


def clean_database(filename: str, path_open: str, path_save: str):
    if not os.path.isfile(filename):
        return None
    xls = pd.ExcelFile(filename)
    sheets = xls.sheet_names
    book = openpyxl.load_workbook(path_open)
    for i in sheets:
        sheet = book[i]
        sheet.delete_rows(2, sheet.max_row - 1)
    book.save(path_save)


def readAllSheets(filename: str):
    """
    Read the .xls file. Store the DataFrame in a dictionary and retrieve all the names of the sheets

    :param filename: DigSilent database file in xls format
    :return: results, sheets
    """

    if not os.path.isfile(filename):
        return None
    xls = pd.ExcelFile(filename)
    sheets = xls.sheet_names
    results = {}
    for sheet in sheets:
        results[sheet] = xls.parse(sheet)
    xls.close()
    return results, sheets

def is_float(variable):
	try:
		float(variable)
		return True
	except:
		return False

def _save_BBDD_temp_xlsx(workbook_DSS: str, elem_select: list, BBDD_OpenDSS: dict, out_path: str, all_elem_DSS: bool):
    """
    Generates the .xlsx file, with xlsx_data format for OpenDSS

    :param workbook_DSS:
    :param elem_select:
    :param BBDD_OpenDSS:
    :param out_path:
    :return:
    """
    if len(elem_select) != 0:
        all_elem_DSS = False
        elem_select.append('Buscoords')

    with pd.ExcelWriter(f'{out_path}\{workbook_DSS}') as writer:
        for elem_name, DF_elem_DSS in BBDD_OpenDSS.items():
            if all_elem_DSS:
                DF_elem_DSS.to_excel(writer, sheet_name=elem_name, index=False)
            else:
                for elem_aux in elem_select:
                    if elem_aux == elem_name:
                        DF_elem_DSS.to_excel(writer, sheet_name=elem_name, index=False)
                if DF_elem_DSS.empty:
                    pass
                else:
                    DF_elem_DSS.to_excel(writer, sheet_name=elem_name, index=False)


def _save_BBDD_xlsx(workbook_DSS: str, BBDD_OpenDSS: dict, out_path: str, add_empty: bool):
    """
    Generates the .xlsx file, with xlsx_data format for OpenDSS

    :param workbook_DSS:
    :param elements_OpenDSS:
    :param BBDD_OpenDSS:
    :param out_path:
    :return:
    """
    try:
        with pd.ExcelWriter(f'{out_path}\{workbook_DSS}') as writer:
            for elem_name, DF_elem_DSS in BBDD_OpenDSS.items():
                if add_empty:
                    DF_elem_DSS.to_excel(writer, sheet_name=elem_name, index=False)
                else:
                    if DF_elem_DSS.empty:
                        pass
                    else:
                        DF_elem_DSS.to_excel(writer, sheet_name=elem_name, index=False)
    except PermissionError:

        writer = pd.ExcelWriter(f'{out_path}\{workbook_DSS}')
        writer.close()


def check_if_element_exists(BBDD_elem_DigS: dict, name_sheets: list):
    """
    According to a list of elements, it checks if they are found in the DigSilent database

    :param BBDD_elem_DigS: DigSilent database file in xls format
    :param name_sheets: List of sheets from DigSilent
    :return: BBDD_DigS
    """
    message = '\n'
    list_DSS = ['General', 'BlkDef', 'ChaRef', 'ElmComp', 'ElmCoup', 'ElmDsl', 'ElmFeeder', 'ElmLne', 'ElmLod',
                'ElmSubstat',
                'ElmNet', 'ElmShnt', 'ElmSym', 'ElmTerm', 'ElmTr2', 'ElmTr3', 'ElmXnet', 'IntFolder', 'IntGrf',
                'IntGrfcon', 'IntGrfnet', 'StaCubic', 'StaSwitch', 'TypGeo', 'TypCon', 'TypLne', 'TypLod', 'TypSym',
                'TypTow', 'TypTr2', 'TypTr3', 'RelFuse', 'IntFolder']

    for name_sheet in list_DSS:
        if len([x for x in [name_sheet] if x in name_sheets]) == 1:
            print(f'Loading {name_sheet} file: With xlsx_data')
            message = message + f'Loading {name_sheet} file: With xlsx_data\n'
        else:
            message = message + f'Loading {name_sheet} file: With xlsx_data\n'
            print(f'Loading {name_sheet} file: Empty')
            if name_sheet == 'General':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(columns=['ID(a:40)', 'Descr(a:40)', 'Val(a:40)'])

            elif name_sheet == 'BlkDef':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)'])

            elif name_sheet == 'ChaRef':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)'])

            elif name_sheet == 'ElmComp':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'outserv(i)',
                             'typ_id(p)'])

            elif name_sheet == 'ElmCoup':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)',
                             'on_off(i)', 'aUsage(a:4)', 'chr_name(a:20)'])

            elif name_sheet == 'ElmDsl':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)', 'outserv(i)'])

            elif name_sheet == 'ElmFeeder':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'obj_id(p)',
                             'iorient(i)', 'i_scale(i)', 'Iset(r)', 'icolor(i)',
                             'outserv(i)'])

            elif name_sheet == 'ElmLne':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'bus1(p)', 'bus2(p)', 'fold_id(p)', 'typ_id(p)', 'dline(r)',
                             'chr_name(a:20)', 'Inom(r)', 'Inom_a(r)', 'outserv(i)', 'rearth(r)', 'c_pcond(p)',
                             'c_ptow(p)', 'c_pcoup(p)', 'cpBranch(p)', 'for_name(a:40)', 'Unom(r)', 'R1(r)', 'X1(r)',
                             'R0(r)', 'X0(r)', 'C1(r)', 'C0(r)', 'B1(r)', 'B0(r)', 'G1(r)', 'G0(r)'])

            elif name_sheet == 'ElmLod':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)', 'chr_name(a:20)',
                             'plini(r)', 'qlini(r)', 'scale0(r)'])

            elif name_sheet == 'ElmLod':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)', 'chr_name(a:20)',
                             'plini(r)', 'qlini(r)', 'scale0(r)'])

            elif name_sheet == 'ElmNet':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'frnom(r)'])

            elif name_sheet == 'ElmShnt':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'chr_name(a:20)', 'shtype(i)', 'ushnm(r)',
                             'qcapn(r)', 'ncapx(i)', 'ncapa(i)', 'outserv(i)'])

            elif name_sheet == 'ElmSym':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)',
                             'ngnum(i)', 'i_mot(i)', 'chr_name(a:20)', 'outserv(i)',
                             'pgini(r)', 'qgini(r)', 'usetp(r)', 'iv_mode(i)',
                             'q_min(r)',
                             'q_max(r)'])

            elif name_sheet == 'ElmTerm':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)', 'iUsage(i)', 'uknom(r)',
                             'chr_name(a:20)', 'outserv(i)', 'GPSlat(r)', 'GPSlon(r)', 'cDisplayName(a)', 'NodeName(a)',
                             'cpBranch(p)', 'cpZone(p)', 'cpArea(p)', 'cpSubstat(p)', 'systype(i)', 'phtech(i)',
                             'unknom(r)', 'cPosLne(r)', 'iminus(i)', 'iEarth(i)', 'cStatName(a)', 'UcteNodeName(a)',
                             'cpGrid(p)', 'cpOwner(p)', 'cpOperator(p)', 'cpSite(p)', 'cpMeteostat(p)', 'cpHeadFold(p)',
                             'ciOutaged(i)', 'ciEnergized(i)', 'ciEarthed(i)', 'cpSupplyTransformer(p)',
                             'cpSupplyTrfStation(p)', 'cpSupplySubstation(p)'])

            elif name_sheet == 'ElmTr2':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)',
                             'outserv(i)', 'nntap(i)', 'sernum(a:20)', 'constr(i)',
                             'chr_name(a:20)'])

            elif name_sheet == 'ElmTr3':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)',
                             'outserv(i)', 'nntap(i)', 'sernum(a:20)', 'constr(i)',
                             'chr_name(a:20)'])

            elif name_sheet == 'ElmXnet':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'outserv(i)', 'snss(r)', 'rntxn(r)',
                             'z2tz1(r)', 'snssmin(r)', 'rntxnmin(r)', 'z2tz1min(r)', 'chr_name(a:20)', 'bustp(a:2)',
                             'pgini(r)', 'qgini(r)', 'phiini(r)', 'usetp(r)'])

            elif name_sheet == 'IntFolder':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)'])

            elif name_sheet == 'IntGrf':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'iCol(i)',
                             'iVis(i)', 'iLevel(i)', 'rCenterX(r)', 'rCenterY(r)',
                             'sSymNam(a:40)',
                             'pDataObj(p)', 'iRot(i)', 'rSizeX(r)', 'rSizeY(r)'])

            elif name_sheet == 'IntGrfcon':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'rX:SIZEROW(i)', 'rX:0(r)', 'rX:1(r)',
                             'rX:2(r)', 'rY:SIZEROW(i)', 'rY:0(r)', 'rY:1(r)', 'rY:2(r)'])

            elif name_sheet == 'IntGrfnet':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'snap_on(i)',
                             'grid_on(i)', 'ortho_on(i)'])

            elif name_sheet == 'StaCubic':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'chr_name(a:20)',
                             'obj_bus(i)', 'obj_id(p)', 'it2p1(i)', 'it2p2(i)', 'it2p3(i)',
                             'cPhInfo(a)', 'nphase(i)'])

            elif name_sheet == 'StaSwitch':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'on_off(i)', 'typ_id(p)', 'iUse(i)'])

            elif name_sheet == 'TypCon':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'uline(r)',
                             'sline(r)', 'ncsub(i)', 'dsubc(r)', 'rpha(r)', 'diaco(r)',
                             'erpha(r)'])

            elif name_sheet == 'TypLne':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'uline(r)', 'sline(r)', 'aohl_(a:3)',
                             'rline(r)', 'xline(r)', 'cline(r)', 'rline0(r)', 'xline0(r)', 'cline0(r)', 'rtemp(r)',
                             'Ithr(r)', 'chr_name(a:20)'])

            elif name_sheet == 'TypLod':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'phtech(i)', 'kpu(r)', 'kqu(r)', 'systp(i)',
                             'aP(r)', 'bP(r)', 'cP(r)', 'aQ(r)', 'bQ(r)', 'cQ(r)', 'kpu0(r)', 'kpu1(r)', 'kqu0(r)',
                             'kqu1(r)', 'chr_name(a:20)'])

            elif name_sheet == 'TypSym':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'sgn(r)', 'ugn(r)', 'cosn(r)', 'xd(r)',
                             'xq(r)', 'xdsss(r)', 'rstr(r)', 'xdsat(r)', 'satur(i)'])

            elif name_sheet == 'TypTow':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'frnom(r)',
                             'nlear(i)', 'nlcir(i)', 'gearth(r)', 'i_mode(i)'])

            elif name_sheet == 'TypTr2':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'strn(r)', 'frnom(r)', 'utrn_h(r)',
                             'utrn_l(r)', 'uktr(r)', 'pcutr(r)', 'uk0tr(r)', 'ur0tr(r)', 'tr2cn_h(a:2)', 'tr2cn_l(a:2)',
                             'nt2ag(r)', 'curmg(r)', 'pfe(r)', 'zx0hl_n(r)', 'tap_side(i)', 'dutap(r)', 'phitr(r)',
                             'nntap0(i)', 'ntpmn(i)', 'ntpmx(i)', 'manuf(a:20)', 'chr_name(a:20)'])

            elif name_sheet == 'TypTr3':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'strn(r)', 'frnom(r)', 'utrn_h(r)',
                             'utrn_l(r)', 'uktr(r)', 'pcutr(r)', 'uk0tr(r)', 'ur0tr(r)', 'tr2cn_h(a:2)', 'tr2cn_l(a:2)',
                             'nt2ag(r)', 'curmg(r)', 'pfe(r)', 'zx0hl_n(r)', 'tap_side(i)', 'dutap(r)', 'phitr(r)',
                             'nntap0(i)', 'ntpmn(i)', 'ntpmx(i)', 'manuf(a:20)', 'chr_name(a:20)'])

            elif name_sheet == 'RelFuse':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'typ_id(p)', 'on_off(i)', 'aUsage(a:4)',
                             'chr_name(a:20)', 'outserv(i)', 'bus1(p)', 'bus2(p)', 'nphase(i)', 'nneutral(i)',
                             'c_type(a)', 'cpOperator(p)', 'ciEnergized(i)'])


            elif name_sheet == 'TypGeo':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'nlear(i)', 'nlcir(i)', 'xy_e:SIZEROW(i)',
                             'xy_e:SIZECOL(i)', 'xy_c:SIZEROW(i)', 'xy_c:SIZECOL(i)', 'xy_c:0:0(r)', 'xy_c:0:1(r)',
                             'xy_c:0:2(r)', 'xy_c:0:3(r)', 'xy_c:0:4(r)', 'xy_c:0:5(r)', 'xy_c:0:6(r)'])


            elif name_sheet == 'ElmSubstat':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)', 'sShort(a:6)', 'Unom(r)', 'GPSlat(r)',
                             'GPSlon(r)'])

            elif name_sheet == 'IntFolder':
                BBDD_elem_DigS[name_sheet] = pd.DataFrame(
                    columns=['ID(a:40)', 'loc_name(a:40)', 'fold_id(p)'])

    return BBDD_elem_DigS, message


def merge_ElmTerm_StaCubic(DataFrame_ElmTerm: pd.DataFrame, DataFrame_StaCubic: pd.DataFrame) -> pd.DataFrame:
    """
    Join ElmTerm and StaCubic

    :param DataFrame_ElmTerm:Excel sheet named ElmTerm from the Digsilent database, stored in a DataFrame
    :param DataFrame_StaCubic:Excel sheet named StaCubic from the Digsilent database, stored in a DataFrame
    :return: merge_StaCubic_ElmTerm
    """
    'Function that eliminates empty spaces and special characters that can cause convergence problems in OpenDSS'
    characters_delete(DataFrame_ElmTerm, 'loc_name(a:40)')

    keys_DataFrame_ElmTerm = DataFrame_ElmTerm.loc[:, ['ID(a:40)', 'loc_name(a:40)', 'uknom(r)', 'ciEnergized(i)']]
    keys_DataFrame_ElmTerm = keys_DataFrame_ElmTerm[
        keys_DataFrame_ElmTerm['ciEnergized(i)'] == 1]  # Filters energized elements

    keys_DataFrame_StaCubic = DataFrame_StaCubic.loc[:, ['ID(a:40)', 'fold_id(p)', 'nphase(i)', 'phase_DSS']]

    'Join the DataFrame_ElmTerm and DataFrame_StaCubic with the key'
    merge_StaCubic_ElmTerm = pd.merge(keys_DataFrame_ElmTerm, keys_DataFrame_StaCubic, how='inner', left_on='ID(a:40)',
                                      right_on='fold_id(p)')
    merge_StaCubic_ElmTerm['Bus_name_DSS'] = merge_StaCubic_ElmTerm['loc_name(a:40)'] + merge_StaCubic_ElmTerm[
        'phase_DSS']
    merge_StaCubic_ElmTerm = merge_StaCubic_ElmTerm[
        ['ID(a:40)_y', 'fold_id(p)', 'loc_name(a:40)', 'uknom(r)', 'nphase(i)', 'Bus_name_DSS', 'phase_DSS']]
    merge_StaCubic_ElmTerm = merge_StaCubic_ElmTerm.rename(columns={'ID(a:40)_y': 'ID(a:40)'})

    return merge_StaCubic_ElmTerm


def merge_ElmTerm_StaCubic_orig(DataFrame_ElmTerm: pd.DataFrame, DataFrame_StaCubic: pd.DataFrame) -> pd.DataFrame:
    """
    Function that eliminates empty spaces and special characters that can cause convergence problems in OpenDSS

    :param DataFrame_ElmTerm: Excel sheet named ElmTerm from the Digsilent database, stored in a DataFrame
    :param DataFrame_StaCubic: Excel sheet named StaCubic from the Digsilent database, stored in a DataFrame
    :return: merge_StaCubic_ElmTerm
    """
    'Function that eliminates empty spaces and special characters that can cause convergence problems in OpenDSS'
    characters_delete(DataFrame_ElmTerm, 'loc_name(a:40)')

    keys_DataFrame_ElmTerm = DataFrame_ElmTerm.loc[:, ['ID(a:40)', 'loc_name(a:40)', 'uknom(r)']]
    keys_DataFrame_StaCubic = DataFrame_StaCubic.loc[:, ['ID(a:40)', 'fold_id(p)', 'nphase(i)', 'phase_DSS']]

    'Join the DataFrame_ElmTerm and DataFrame_StaCubic with the key'
    merge_StaCubic_ElmTerm = pd.merge(keys_DataFrame_ElmTerm, keys_DataFrame_StaCubic, how='inner', left_on='ID(a:40)',
                                      right_on='fold_id(p)')
    merge_StaCubic_ElmTerm['Bus_name_DSS'] = merge_StaCubic_ElmTerm["loc_name(a:40)"] + merge_StaCubic_ElmTerm[
        "phase_DSS"]
    merge_StaCubic_ElmTerm = merge_StaCubic_ElmTerm[
        ['ID(a:40)_y', 'fold_id(p)', 'loc_name(a:40)', 'uknom(r)', 'nphase(i)', 'Bus_name_DSS']]
    merge_StaCubic_ElmTerm = merge_StaCubic_ElmTerm.rename(columns={'ID(a:40)_y': 'ID(a:40)'})

    return merge_StaCubic_ElmTerm


def name_folders_elements(dict_df_DigS: dict) -> dict:
    '''
    Add to Id_elem the name of the folder that belongs to the Digsilent database

    :param dict_df_DigS:DigSilent database file in xls format
    :return: dict_df_DigS
    '''

    DataFrame_IntFolder = dict_df_DigS['IntFolder']
    if DataFrame_IntFolder.empty == True:
        pass
    else:
        if dict_df_DigS['TypCon'].empty == True:
            pass
        else:
            dict_df_DigS['TypCon'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypCon',
                                                                 dict_df_DigS=dict_df_DigS)
        if dict_df_DigS['TypTow'].empty == True:
            pass
        else:
            dict_df_DigS['TypTow'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypTow',
                                                                 dict_df_DigS=dict_df_DigS)
        if dict_df_DigS['TypGeo'].empty == True:
            pass
        else:
            dict_df_DigS['TypGeo'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypGeo',
                                                                 dict_df_DigS=dict_df_DigS)
        if dict_df_DigS['TypLne'].empty == True:
            pass
        else:
            dict_df_DigS['TypLne'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypLne',
                                                                 dict_df_DigS=dict_df_DigS)
        if dict_df_DigS['TypTr2'].empty == True:
            pass
        else:
            dict_df_DigS['TypTr2'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypTr2',
                                                                 dict_df_DigS=dict_df_DigS)
        if dict_df_DigS['TypTr3'].empty == True:
            pass
        else:
            dict_df_DigS['TypTr3'] = merge_name_folders_elements(DataFrame_IntFolder=DataFrame_IntFolder,
                                                                 name_elem='TypTr3',
                                                                 dict_df_DigS=dict_df_DigS)
    return dict_df_DigS


def merge_name_folders_elements(DataFrame_IntFolder: pd.DataFrame, name_elem: str, dict_df_DigS: dict):
    '''
    Join the Id_elem with the name of the folder it belongs to

    :param DataFrame_IntFolder:DataFrame -> IntFolder
    :param name_elem:Type the element
    :param dict_df_DigS:DigSilent database file in xls format
    :return: DataFrame_elem
    '''

    DataFrame_IntFolder = DataFrame_IntFolder[['ID(a:40)', 'loc_name(a:40)']]
    DataFrame_elem = dict_df_DigS[name_elem]
    merge_TypCon_IntFolder = pd.merge(DataFrame_elem[['loc_name(a:40)', 'fold_id(p)']], DataFrame_IntFolder,
                                      how='left', left_on='fold_id(p)', right_on='ID(a:40)',
                                      suffixes=('_x', '_y')).fillna('')

    for index, row in merge_TypCon_IntFolder.iterrows():
        if merge_TypCon_IntFolder['loc_name(a:40)_y'][index] == '':
            DataFrame_elem['loc_name(a:40)'][index] = merge_TypCon_IntFolder['loc_name(a:40)_x'][index]
        else:
            DataFrame_elem['loc_name(a:40)'][index] = merge_TypCon_IntFolder['loc_name(a:40)_x'][index] + f' ' + \
                                                      merge_TypCon_IntFolder['loc_name(a:40)_y'][index]
    return DataFrame_elem


def file_logging_info(logfilename: str, message: str):
    logging.basicConfig(filename=f'{logfilename}.log',
                        format='%(asctime)s %(levelname)-8s %(message)s',
                        level=logging.INFO,
                        datefmt='%Y-%m-%d %H:%M:%S')

    logging.info(message)


def check_version_of_BBDD(BBDD_DigS: dict, name_sheets: list):
    version = int(BBDD_DigS['General']['Val(a:40)'][0])

    if version == 6:
        for name in name_sheets:
            BBDD_DigS[name] = BBDD_DigS[name].rename(columns={'FID(a:40)': 'ID(a:40)'})

    elif version == 5:
        pass
    else:
        print('Export the database in Digsilent in version 5 (recommended) or 6')
        exit()

    return BBDD_DigS, name_sheets



# -*- coding: utf-8 -*-
# @Time    : 18/11/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm
from typing import Dict, Any, Union

import pandas as pd
from numpy import ndarray
from pandas import Series, DataFrame
from pandas.core.arrays import ExtensionArray

from openpy_fx_tools_dss.helper_functions import *
from openpy_fx_tools_dss.interface_dss import dss, drt

def Buscoords_DSS(BBDD_OpenDSS: dict, DSS_elem_list: list, opt: str):

    coord = dict()
    all_bus_names = dss.circuit_all_bus_names()

    for name in all_bus_names:
        coord[name] = dict()
        dss.circuit_set_active_bus(name)
        if opt == 'XY':
            coord[name]['Y'] = dss.bus_read_y()
            coord[name]['X'] = dss.bus_read_x()
        elif opt == 'LatLong':
            coord[name]['Long'] = dss.bus_read_y()
            coord[name]['Lat'] = dss.bus_read_x()
        '''
        # Possible use of code in other type of coordinates 
        if opt == 'XY':
            coord[name]['Y'] = dss.bus_read_y()
            coord[name]['X'] = dss.bus_read_x()
        else:
            coord[name]['Long'] = dss.bus_read_longitude()
            coord[name]['Lat'] = dss.bus_read_latitude() 
        '''
    df_Buscoords_DSS = pd.DataFrame(pd.DataFrame(coord).T).reset_index().rename(columns={'index': f'Bus name'})
    if df_Buscoords_DSS.empty:
        if opt == 'XY':
            df_Buscoords_DSS = pd.DataFrame(columns=['Bus name', 'Y', 'X'])
        elif opt == 'LatLong':
            df_Buscoords_DSS = pd.DataFrame(columns=['Bus name', 'Long', 'Lat'])
    BBDD_OpenDSS['Buscoords'] = df_Buscoords_DSS
    DSS_elem_list.append('Buscoords')
    return BBDD_OpenDSS, DSS_elem_list





# -*- coding: utf-8 -*-
# @Time    : 8/23/2022 3:41 PM
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : create_file_xlsx.py
# @Software: PyCharm

import logging
import pandas as pd

from ...interface_dss import dss, drt
from ...helper_functions import _save_BBDD_xlsx, _save_BBDD_temp_xlsx, is_float
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.Other_elem_DSS import Other_MTY, Other_Def_Value, Other_ORD, list_Other_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.General_elem_DSS import General_MTY, General_Def_Value, General_ORD, list_General_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.PD_elem_DSS import PD_elements_MTY, PD_elem_Def_Value, PD_elements_ORD, list_PD_elements_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.PC_elem_DSS import PC_elements_MTY, PC_elem_Def_Value, PC_elements_ORD, list_PC_elements_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.Controls_elem_DSS import Controls_MTY, Controls_Def_Value, Controls_ORD, list_Controls_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.Meters_elem_DSS import Meters_MTY, Meters_Def_Value, Meters_ORD, list_Meters_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.Voltagebases_DSS import Voltagebases_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.xlsx_DSS_xlsx.Types_elem_DSS_to_xlxs.Buscoords import Buscoords_DSS
from openpy_fx_tools_dss.IMEX_to_DSS.GIS_info_DSS.Update_element_coordinates import Update_coordinates_for_GIS

from openpy_fx_tools_dss.logg_print_alert import logg_alert

log_py = logging.getLogger(__name__)

list_General = list()
list_Other = list()
list_PD_elements = list()
list_PC_elements = list()
list_Controls = list()
list_Meters = list()

def _Create_template_xlsx(path_save: str, prj_name: str, elem: list, all_elem: bool):
    n_elem_add = {"General": 0, "Other": 0, "PD_elem": 0, "PC_elem": 0, "Controls": 0, "Meters": 0}

    DSS_elem_list = list()
    BBDD_OpenDSS = dict()
    list_no_class = list()

    drt.run_command('ClearAll')
    drt.run_command(f'new circuit.{prj_name}')

    list_no = ['Solution']
    for ClassName in dss.dss_classes():
        if len([x for x in [ClassName] if x in list_no]) != 1:
            BBDD_OpenDSS, DSS_elem_list, n_elem_add = _Add_BBDD_list_DSS(
                BBDD_OpenDSS=BBDD_OpenDSS,
                DSS_elem_list=DSS_elem_list,
                ClassName=ClassName,
                dict_class=n_elem_add
            )
    BBDD_OpenDSS, DSS_elem_list = Buscoords_DSS(
        BBDD_OpenDSS=BBDD_OpenDSS,
        DSS_elem_list=DSS_elem_list
    )

    BBDD_OpenDSS = _check_DSS_default_values(BBDD_OpenDSS=BBDD_OpenDSS)

    xlsx_name_DSS = f'BBDD_DSS_{prj_name}.xlsx'
    _save_BBDD_temp_xlsx(
        workbook_DSS=xlsx_name_DSS,
        elem_select=elem,
        BBDD_OpenDSS=BBDD_OpenDSS,
        out_path=path_save,
        all_elem_DSS=all_elem
    )


def _Create_DSS_to_xlsx_files(
        DSS_file: str, path_save: str, prj_name: str, add_empty: bool, coords: str):

    n_elem_dss = {
        "General": len(list_General_DSS),
        "Other": len(list_Other_DSS),
        "PD_elem": len(list_PD_elements_DSS),
        "PC_elem": len(list_PC_elements_DSS),
        "Controls": len(list_Controls_DSS),
        "Meters": len(list_Meters_DSS)
    }
    n_elem_add = {"General": 0, "Other": 0, "PD_elem": 0, "PC_elem": 0, "Controls": 0, "Meters": 0}
    DSS_elem_list = list()
    BBDD_OpenDSS = dict()
    list_no_class = list()
    dss.text("ClearAll")
    dss.text(f"compile [{DSS_file}]")
    drt.run_command(f"compile [{DSS_file}]")
    list_no = ['Solution']
    for ClassName in dss.dss_classes():
        if len([x for x in [ClassName] if x in list_no]) != 1:
            BBDD_OpenDSS, DSS_elem_list, n_elem_add = _Add_BBDD_list_DSS(
                BBDD_OpenDSS=BBDD_OpenDSS,
                DSS_elem_list=DSS_elem_list,
                ClassName=ClassName,
                dict_class=n_elem_add
            )
        else:
            pass
    #Read and create BusCoords file
    BBDD_OpenDSS, DSS_elem_list = Buscoords_DSS(
        BBDD_OpenDSS=BBDD_OpenDSS,
        DSS_elem_list=DSS_elem_list,
        opt=coords
    )
    if logg_alert.export_csv:
        BBDD_OpenDSS['Buscoords'].to_csv(f'{path_save}\Buscoords_{prj_name}.csv', header=False, index=False)
        BBDD_OpenDSS.pop('Buscoords')

    BBDD_OpenDSS = _check_DSS_default_values(BBDD_OpenDSS=BBDD_OpenDSS)
    xlsx_name_DSS = f'BBDD_DSS_{prj_name}.xlsx'
    _save_BBDD_xlsx(
        workbook_DSS=xlsx_name_DSS,
        BBDD_OpenDSS=BBDD_OpenDSS,
        out_path=path_save,
        add_empty=add_empty
    )

    logg_alert.update_logg_file('_' * 64, 1)
    logg_alert.update_logg_file(f'Created and saved the {xlsx_name_DSS} file in the path:', 2, log_py)
    logg_alert.update_logg_file(f'{path_save}', 0, log_py)

    if logg_alert.GIS_Info:
        Update_coordinates_for_GIS(prj_name=prj_name, DSS_file=DSS_file, save_path=path_save)




def _check_DSS_default_values(BBDD_OpenDSS: dict):
    for ClassName in BBDD_OpenDSS.keys():
        if len([x for x in [ClassName] if x in list_General_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = General_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
        if len([x for x in [ClassName] if x in list_Other_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = Other_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
        if len([x for x in [ClassName] if x in list_PD_elements_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = PD_elem_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
        if len([x for x in [ClassName] if x in list_PC_elements_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = PC_elem_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
        if len([x for x in [ClassName] if x in list_Controls_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = Controls_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
        if len([x for x in [ClassName] if x in list_Meters_DSS]) == 1:
            BBDD_OpenDSS[ClassName] = Meters_Def_Value(DF_elem_DSS=BBDD_OpenDSS[ClassName], name_class=ClassName)
    return BBDD_OpenDSS

def ClassName_to_DataFrame(ClassName: str, transform_string=None, clean_data=None):
    if transform_string is None:
        transform_string = _evaluate_expression
    if clean_data is None:
        clean_data = _clean_data
    data = dict()
    drt.Circuit.SetActiveClass("{class_name}".format(class_name=ClassName))
    for element in drt.ActiveClass.AllNames():
        name = "{element}".format(element=element)
        drt.ActiveClass.Name(element)
        data[name] = dict()
        for i, n in enumerate(drt.Element.AllPropertyNames()):
            # use 1-based index for compatibility with previous versions
            PropertyName = drt.Element.AllPropertyNames()[i]
            if PropertyName == 'WdgCurrents':
                string = ''
            else:
                string = drt.Properties.Value(str(i + 1))
            string = transform_string(string)
            if type(string) == list:
                string = str(string).replace("'", "")
                if string == "[]":
                    string = ""
            if type(string) == tuple:
                string = str(list(string)).replace("'", "")
                if string == "[]":
                    string = ""
            if PropertyName == 'bus1':
                string = str(string)
            if PropertyName == 'bus2':
                string = str(string)
            if PropertyName == 'linecode':
                if is_float(string):
                    string = str(int(string))
                else:
                    string = str(string)
            data[name][n] = string
    data = clean_data(data, ClassName)
    df_class = pd.DataFrame(pd.DataFrame(data).T).reset_index().rename(columns={'index': f'Id_{ClassName}'})
    return df_class



def _Add_BBDD_list_DSS(BBDD_OpenDSS: dict, DSS_elem_list: list, ClassName: str, dict_class: dict):
    elem_no_drt = ['WindGen', 'Generic5', 'FMonitor']
    PC_elem_no_drt = ['WindGen', 'Generic5']
    if len([x for x in [ClassName] if x in elem_no_drt]) == 1:
        dss.circuit_set_active_class(ClassName)
        if len([x for x in [ClassName] if x in PC_elem_no_drt]) == 1:
            BBDD_OpenDSS, DSS_elem_list = PC_elements_MTY(
                BBDD_elem_DSS=BBDD_OpenDSS,
                DSS_elem_list=DSS_elem_list,
                name_class=ClassName
            )
            list_PC_elements.append(ClassName)
            dict_class['PC_elem'] += 1
        if ClassName == 'FMonitor':
            BBDD_OpenDSS, DSS_elem_list = Meters_MTY(
                BBDD_elem_DSS=BBDD_OpenDSS,
                DSS_elem_list=DSS_elem_list,
                name_class=ClassName
            )
            list_Meters.append(ClassName)
            dict_class['Meters'] += 1
    else:
        df_ClassName_elem = ClassName_to_DataFrame(ClassName)
        if len([x for x in [ClassName] if x in list_General_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = General_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = General_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)

            list_General.append(ClassName)
            dict_class['General'] += 1
        if len([x for x in [ClassName] if x in list_Other_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = Other_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = Other_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)
            list_Other.append(ClassName)
            dict_class['Other'] += 1
        if len([x for x in [ClassName] if x in list_PD_elements_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = PD_elements_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = PD_elements_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)

            list_PD_elements.append(ClassName)
            dict_class['PD_elem'] += 1
        if len([x for x in [ClassName] if x in list_PC_elements_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = PC_elements_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = PC_elements_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)
            list_PC_elements.append(ClassName)
            dict_class['PC_elem'] += 1
        if len([x for x in [ClassName] if x in list_Controls_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = Controls_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = Controls_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)
            list_Controls.append(ClassName)
            dict_class['Controls'] += 1
        if len([x for x in [ClassName] if x in list_Meters_DSS]) == 1:
            if df_ClassName_elem.empty:
                BBDD_OpenDSS, DSS_elem_list = Meters_MTY(
                    BBDD_elem_DSS=BBDD_OpenDSS,
                    DSS_elem_list=DSS_elem_list,
                    name_class=ClassName
                )
            else:
                BBDD_OpenDSS[ClassName] = Meters_ORD(
                    DF_elem_DSS=df_ClassName_elem,
                    name_class=ClassName
                )
                DSS_elem_list.append(ClassName)
            list_Meters.append(ClassName)
            dict_class['Meters'] += 1
        BBDD_OpenDSS['Voltagebases'] = Voltagebases_DSS()
        DSS_elem_list.append('Voltagebases')
    return BBDD_OpenDSS, DSS_elem_list, dict_class

def _evaluate_expression(string):
    list_except = ['none', '----', 'No', 'oh', '', '0', "-1", "shell"]
    list_units = ['mi', 'kft', 'km', 'm', 'Ft', 'in', 'cm']
    if "[" in string and "]" in string:
        str_mdf = [
            _evaluate_expression(x.strip())
            for x in string.replace("[", "").replace("]", "").split(",")
            if x.strip() != ""
        ]
        return str_mdf
    elif len([x for x in [string] if x in list_except]) == 1:
        str_mdf = ''
        return str_mdf
    elif len([x for x in [string] if x in list_units]) == 1:
        return string
    elif string.startswith("(") and string.endswith(")"):
        str_mdf = tuple(
            _evaluate_expression(x.strip())
            for x in string.replace("(", "").replace(")", "").split(",")
            if x.strip() != ""
        )
        return str_mdf
    elif string.lower() == "true":
        return True
    elif string.lower() == "false":
        return False
    elif "|" in string:
        return string
    elif string.startswith("'") and string.endswith("'"):
        print('here')
    elif type(string) == list:
        return string
    else:
        try:
            string = float(string)
            return string
        except ValueError:
            return string

def _clean_data(data, class_name):
    for element in drt.ActiveClass.AllNames():
        name = "{element}".format(element=element)
        drt.ActiveClass.Name(element)

        if "nconds" in drt.Element.AllPropertyNames():
            nconds = int(data[name]["nconds"])
            x = []
            h = []
            units = []

            for cond in range(1, nconds + 1):
                drt.run_command("{name}.cond={cond}".format(name=name, cond=cond))
                x.append(float(drt.run_command("? {name}.x".format(name=name))))
                h.append(float(drt.run_command("? {name}.h".format(name=name))))
                units.append(drt.run_command("? {name}.units".format(name=name)))

            data[name]["x"] = x
            data[name]["h"] = h
            data[name]["units"] = units

        if class_name == 'Vsource':
            list_AllPropertyNames = drt.Element.AllPropertyNames()
            pos = data[name]['bus1'].find('.')
            ph_1 = data[name]['bus1'][pos:]
            if ph_1.find('0') == -1:
                pass
            else:
                if int(data[name]['phases']) == 3:
                    data[name]["bus1"] = data[name]["bus1"][:pos]

        else:
            list_AllPropertyNames = drt.Element.AllPropertyNames()
            ph_aux = ['nphases', 'phases']
            for k in ph_aux:
                if len([x for x in [k] if x in list_AllPropertyNames]) == 1:
                    if 'bus1' in drt.Element.AllPropertyNames():
                        pos = data[name]['bus1'].find('.')
                        ph_1 = data[name]['bus1'][pos:]
                        if ph_1.find('0') == -1:
                            pass
                        else:
                            if int(data[name][k]) == 3:
                                data[name]["bus1"] = data[name]["bus1"][:pos] + '.1.2.3' + ph_1

                    if 'bus2' in drt.Element.AllPropertyNames():
                        if class_name != 'Capacitor':
                            pos = data[name]['bus2'].find('.')
                            ph_2 = data[name]['bus2'][pos:]
                            if ph_2.find('0') == -1:
                                pass
                            else:
                                if int(data[name][k]) == 3:
                                    data[name]["bus2"] = data[name]["bus2"][:pos] + '.1.2.3' + ph_2

                    if 'conns' in drt.Element.AllPropertyNames():
                        pos = data[name]['buses']
                        aux1 = pos.find('[')
                        aux2 = pos.find(',')
                        aux3 = pos.find(']')

                        list_buses = [pos[aux1 + 1:aux2], pos[aux2 + 1:aux3]]

                        list_nbus = list()

                        for bus in list_buses:
                            pos = bus.find('.')
                            ph_1 = bus[pos:]
                            if ph_1.find('0') == -1:
                                list_nbus.append(bus)
                            else:
                                if int(data[name]['phases']) == 3:
                                    list_nbus.append(bus[:pos] + '.1.2.3' + ph_1)

                        data[name]['buses'] = f'[{list_nbus[0]}, {list_nbus[1]}]'


    return data


# -*- coding: utf-8 -*-
# @Time    : 19/10/2021
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import pandas as pd
from io import open
import numpy as np
import openpyxl
import os
import glob
import shutil

def create_folder(name_folder: str):
    os.mkdir(name_folder)

def copy_file(file: str, root_file_path: str, name_folder: str):
    '''
    :param file:
    :param root_file_path:
    :param name_folder:
    :return:
    '''
    shutil.copyfile(src=root_file_path+f'/{file}', dst=root_file_path+f'/{name_folder}')


#def create_DSS(name_dss:str, file_BBDD:str, root_path:str, name_folder:str):+
#create_DSS(name_dss=export_name, file_BBDD=f'{address_saves_DSS_files}\BBDD_DSS_{export_name}.xlsx', path_save=address_saves_DSS_files)

def create_DSS(name_dss: str, address_saves: str):
    """

    :param name_dss:
    :param address_saves:
    :return:
    """
    file_BBDD = f'{address_saves}\BBDD_DSS_{name_dss}.xlsx'
    directory = os.chdir(address_saves)
    create_scrips_base_dss(name_dss=name_dss, workbook=file_BBDD, ruta_archivo=address_saves)

def create_scrips_base_dss(name_dss:str, workbook:str, ruta_archivo:str):
    '''
    :param name_dss:
    :param workbook:
    :param ruta_archivo:
    :return OpenDSS scripts in the specified path_save

    function that creates the OpenDSS scripts with the information from the database
    '''

    delete_all_files(ruta_archivo)
    create_master_base_dss(name_dss, workbook, ruta_archivo)
    for element in check_BBDD(workbook):
        create_element_base_dss(name_dss, element, workbook)
    print('Data loaded and .DSS files created')

def create_cases(name_dss:str, file_BBDD:str, root_path:str,name_folder:str):
    '''
    :param name_dss:
    :param file:
    :param root_path:
    :param name_folder:
    :return:
    '''
    indicator = os.path.exists(name_folder)
    if indicator == True:
        shutil.rmtree(name_folder)
        create_folder(name_folder)
        delete_files_DSS()
        root_file_path = root_path+f'/{file_BBDD}'
        root_file_path_csv = root_path + f'/Buscoods_{name_dss}.csv'
        name_dss_aux = name_dss + f'_{name_folder}'
        create_scrips_dss(name_dss_aux, file_BBDD, root_file_path)
        move_files_DSS(root_path, root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path, dst=root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path_csv, dst=root_path + f'/{name_folder}')
        os.rename(root_path + f'/{name_folder}' + f'/{file_BBDD}', root_path+ f'/{name_folder}' + f'/BBDD_{name_folder}.xlsx')
        os.rename(root_path + f'/{name_folder}' + f'/Buscoods_{name_dss}.csv', root_path + f'/{name_folder}' + f'/Buscoods_{name_dss}_{name_folder}.csv')

    elif indicator == False:
        create_folder(name_folder)
        delete_files_DSS()
        root_file_path = root_path + f'/{file_BBDD}'
        root_file_path_csv = root_path + f'/Buscoods_{name_dss}.csv'
        name_dss_aux = name_dss + f'_{name_folder}'
        create_scrips_dss(name_dss_aux, file_BBDD, root_file_path)
        move_files_DSS(root_path, root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path, dst=root_path + f'/{name_folder}')
        shutil.copy(src=root_file_path_csv, dst=root_path + f'/{name_folder}')
        os.rename(root_path + f'/{name_folder}' + f'/{file_BBDD}', root_path + f'/{name_folder}' + f'/BBDD_{name_folder}.xlsx')
        os.rename(root_path + f'/{name_folder}' + f'/Buscoods_{name_dss}.csv', root_path + f'/{name_folder}' + f'/Buscoods_{name_dss}_{name_folder}.csv')

    else:
        print('Cannot perform the operation try another name')

def file_path(workbook):
    return read_direction()+workbook

def read_direction():
    file_path = os.path.dirname(os.path.abspath(__file__))
    file_path = file_path.replace("\\", "/")
    return file_path

def move_files_DSS(source_address, destination_address):
    '''
    :return:
    Delete existing .DSS files in the specified path_save
    '''
    direction = read_direction()
    dss_files = glob.glob(direction+'/*.dss')
    dss_files2 = glob.glob('/*.dss')
    for dss_file in dss_files:
        shutil.move(dss_file, destination_address)

def delete_all_files(address):
    '''
    :return:
    Delete existing .DSS files in the specified path_save
    '''
    direction = address
    #dss
    dss_files = glob.glob(direction+'/*.dss')
    for dss_file in dss_files:
        try:
            os.remove(dss_file)
        except OSError as e:
            print(f"Error:{e.strerror}")



def delete_files_DSS():
    '''
    :return:
    Delete existing .DSS files in the specified path_save
    '''
    direction = read_direction()
    dss_files = glob.glob(direction+'/*.dss')
    for dss_file in dss_files:
        try:
            os.remove(dss_file)
        except OSError as e:
            print(f"Error:{e.strerror}")

def master_content(name_dss:str, list_elements:list):
    '''
    :param name_dss:
    :param list_elements:
    :return:
    '''
    content = 'Clear\n'\
              '\n'\
              f'New Circuit.{name_dss}\n'\
              '\n'
    aux = ''
    content_aux = ''
    for element in list_elements:
        content_aux = f'Redirect {element}_{name_dss}.dss\n'
        aux = aux + content_aux
    aux = content + aux

    aux_1 = '\n'\
            f'LatLongCoords Buscoods_{name_dss}.csv'\
            '\n'

    aux_2 = '\n'\
            'solve'

    aux = aux + aux_1 + aux_2
    return aux


def create_master_base_dss(name_dss:str, workbook_name:str, dir:str):
    '''
    :param name_dss:
    :param workbook_name:
    :return:
    '''
    os.chdir(dir)
    os.getcwd()
    list_elements = check_BBDD(workbook_name)
    master_dss = open(f'Master_{name_dss}.dss','w')
    content = master_content(name_dss, list_elements)
    master_dss.write(content)
    master_dss.close()

def create_master_dss(name_dss:str, workbook_name:str):
    '''
    :param name_dss:
    :param workbook_name:
    :return:
    '''
    list_elements = check_BBDD(workbook_name)
    master_dss = open(f'Master_{name_dss}.dss','w')
    content = master_content(name_dss, list_elements)
    master_dss.write(content)
    master_dss.close()


def check_BBDD(workbook_name:str):
    '''
    :param workbook_name:
    :return:
    '''
    workbook = openpyxl.load_workbook(workbook_name, read_only=True)
    name_sheets = workbook.sheetnames
    list_elements = []
    for name_sheet in name_sheets:
        df_sheet = pd.read_excel(workbook_name, sheet_name=name_sheet)
        if len(df_sheet) >= 1:
            list_elements.append(name_sheet)
        else:
            pass
    return list_elements

def create_element_base_dss(name_dss:str, element:str, direction:str):
    '''
    :param name_dss:
    :param element:
    :param direction:
    :return:
    '''
    df_element = pd.read_excel(direction, sheet_name='{}'.format(element))
    dict_element = df_element.to_dict()
    keys = [*dict_element]
    script = ''
    aux = 0
    list_aux = []
    for n in range(len(dict_element['Id_{}'.format(element)])):
        for key in keys:
            if key == 'Id_{}'.format(element):
                if dict_element[key][aux] == 'source':
                    script = 'Edit "{}.{}" '.format(element, dict_element[key][aux])
                else:
                    if element == 'Switch':
                        element_aux = 'Line'
                        script = 'New "{}.{}" '.format(element_aux, dict_element[key][aux])
                    elif element == 'Voltagebases':
                        script = f'Set Voltagebases = {dict_element[key][aux]}\n' \
                                 'calcv\n'
                    else:
                        script = 'New "{}.{}" '.format(element, dict_element[key][aux])

                list_aux.append(script)
            else:
                if aux <= len(dict_element[key]):
                    if str(dict_element[key][aux]) != 'nan':
                        script = '{}={} '.format(key, dict_element[key][aux])
                        list_aux.append(script)
                    else:
                        pass
        list_aux.append('\n')

        aux += 1
    element_content = '{}'.format(''.join(list_aux))
    element_dss = open('{}_{}.dss'.format(element, name_dss), 'w')
    element_dss.write(element_content)
    element_dss.close()

def create_element_dss(name_dss:str, element:str, direction:str):
    '''
    :param name_dss:
    :param element:
    :param direction:
    :return:
    '''
    df_element = pd.read_excel(direction, sheet_name='{}'.format(element))
    dict_element = df_element.to_dict()
    keys = [*dict_element]
    script = ''
    aux = 0
    list_aux = []
    for n in range(len(dict_element['Id_{}'.format(element)])):
        for key in keys:
            if key == 'Id_{}'.format(element):
                if dict_element[key][aux] == 'source':
                    script = 'Edit "{}.{}" '.format(element, dict_element[key][aux])
                else:
                    if element == 'Switch':
                        element_aux = 'Line'
                        script = 'New "{}.{}" '.format(element_aux, dict_element[key][aux])
                    elif element == 'Voltagebases':
                        script = f'Set Voltagebases = {dict_element[key][aux]}\n' \
                                 'calcv\n'
                    else:
                        script = 'New "{}.{}" '.format(element, dict_element[key][aux])

                list_aux.append(script)
            else:
                if aux <= len(dict_element[key]):
                    if str(dict_element[key][aux]) != 'nan':
                        script = '{}={} '.format(key, dict_element[key][aux])
                        list_aux.append(script)
                    else:
                        pass
        list_aux.append('\n')

        aux += 1
    element_content = '{}'.format(''.join(list_aux))
    element_dss = open('{}_{}.dss'.format(element, name_dss), 'w')
    element_dss.write(element_content)
    element_dss.close()


def create_scrips_dss(name_dss:str, workbook:str, ruta_archivo:str):
    '''
    :param name_dss:
    :param workbook:
    :param ruta_archivo:
    :return OpenDSS scripts in the specified path_save

    function that creates the OpenDSS scripts with the information from the database
    '''
    delete_files_DSS()
    create_master_dss(name_dss, workbook)
    for element in check_BBDD(workbook):
        create_element_dss(name_dss, element, ruta_archivo)
    print('Data loaded and .DSS files created')

def create_scripts_OpenDSS(name_dss, workbook, ruta_raiz, name_folder):
    create_cases(name_dss, workbook, ruta_raiz, name_folder)


# -*- coding: utf-8 -*-
# @Time    : 16/11/2022
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : Voltagebases_DSS.py
# @Software: PyCharm


import warnings
import pandas as pd
from openpy_fx_tools_dss.interface_dss import dss

warnings.simplefilter(action='ignore', category=Warning)

def Voltagebases_DSS() -> pd.DataFrame:
    list_aux = dss.settings_read_voltage_bases()
    data = str(list_aux).replace("[", "").replace("]", "")
    dic = {'Id_Voltagebases': data}
    df_VoltBase_DSS = pd.DataFrame(columns=['Id_Voltagebases'])
    df_VoltBase_DSS = df_VoltBase_DSS.append({'Id_Voltagebases': list_aux}, ignore_index=True)

    return df_VoltBase_DSS

